#include <userint.h>
#include <utility.h>
#include <formatio.h>
#include <ansi_c.h>
#include "lc8901a.h"
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include "uklon_ui.h"

#define NDIS 21
#define NSCA 22
#define NTGG 23

int nc[]={P1_NC0, P1_NC1, P1_NC2, P1_NC3, 
          P1_NC4, P1_NC5, P1_NC6, P1_NC7,
          P1_NC8, P1_NC9, P1_NC10,P1_NC11,
          P1_NC12,P1_NC13,P1_NC14,P1_NC15 } ;


//********************************************************************

int init_CAMAC_GPIB ( ViSession CAMACgpib ) {

  lc8901a_InitClearInhibit (CAMACgpib, 0);
  lc8901a_Write (CAMACgpib, 9, 0, NTGG, 0); /* init TGG */
  lc8901a_Write (CAMACgpib, 16, 0, NTGG, 1000); /* Set preset counting value */
  lc8901a_Write (CAMACgpib, 17, 0, NTGG, 0x2); /* Set Load&Clock Modes */
  lc8901a_Write (CAMACgpib, 16, 0, NDIS, 0xffff); /* enable all ch. */
  lc8901a_Write (CAMACgpib, 26, 0, NDIS, 0);

  return 0 ;
}

//********************************************************************

int plot_graph ( int p1h, int logsc, int data[16] ) {
  int max, max_I2 ;
  double plot_data[16], plot_max ;
  int ch ;

  max = 100 ;
  for ( ch=0 ; ch<16 ; ch ++ ) {
    if ( data[ch]>max ) max=data[ch] ;
  }

  DeleteGraphPlot (p1h, P1_HISTO, -1, VAL_IMMEDIATE_DRAW);
  
  max = ((max*1.2)/1000)*1000 ;
  GetCtrlVal (p1h, P1_I2, &max_I2 ) ;
  if ( max_I2>0 ) max=max_I2 ;
  
  if ( logsc ) {
      		
    for ( ch=0 ; ch<16 ; ch ++ ) {  
	  plot_data[ch] = log10((double)data[ch]) ; 
	  plot_max = log10((double)max) ;
	}
	  
	SetAxisScalingMode (p1h, P1_HISTO, VAL_LEFT_YAXIS, VAL_MANUAL, 0.0, plot_max);
	PlotY (p1h, P1_HISTO, plot_data, 16, VAL_DOUBLE, VAL_VERTICAL_BAR,
		   VAL_NO_POINT, VAL_SOLID, 1, VAL_DK_BLUE);
      
  
  } else {
    SetAxisScalingMode (p1h, P1_HISTO, VAL_LEFT_YAXIS, VAL_MANUAL, 0.0, max);
    PlotY (p1h, P1_HISTO, data, 16, VAL_INTEGER, VAL_VERTICAL_BAR,
	  	   VAL_NO_POINT, VAL_SOLID, 1, VAL_DK_BLUE);
     
    for ( ch=0; ch<16; ch++ ) {
      SetCtrlVal (p1h, nc[ch], data[ch] ) ;
    }
  }


  return 0 ;
}


//********************************************************************
//********************************************************************

int __stdcall WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
					   LPSTR lpszCmdLine, int nCmdShow)
{
  ViSession CAMACgpib;
  int rph, rID, p1h, p2h ;
  int data[16], sdata[10][32], sum, max, max_I2=0 ;
  double plot_data[16], splot_data[10][32], plot_max ;
  int thr_set, time_set;
  int ch, i, j;
  ViBoolean Xr, Qr;
  FILE *fp ;
  char filename[256] ;
  int logscale ;
  int os[10]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1} ;
  
  logscale = 0 ;

  if (InitCVIRTE (hInstance, 0, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
      return -1;    /* out of memory */

  for ( i=0; i<16; data[i++]=0 ) ;
  for ( i=0; i<10; i++ ) for ( j=0; j<32; sdata[i][j++]=0 );

                                 
  lc8901a_init ("GPIB0::2", 0, 2, &CAMACgpib);
  init_CAMAC_GPIB ( CAMACgpib ) ;   
  
  p1h = LoadPanel (0, "uklon_ui.uir", P1);
//  p2h = LoadPanel (0, "uklon_ui.uir", P2);
  DisplayPanel (p1h);
//  DisplayPanel (p2h);

  GetUserEvent (1, &rph, &rID);


  while ((rID!=P1_QUIT)||(rph!=p1h))
  {
    if ( rph == p1h ) {
  		if ( rID==P1_START ) {
	      GetCtrlVal (p1h, P1_THR, &thr_set ) ;   /* read desired thr. */  
    	  lc8901a_Write (CAMACgpib, 17, 0, NDIS, thr_set);   /* set the threshold */
		  lc8901a_Write (CAMACgpib, 17, 1, NDIS, 0);         /*					*/
		
	      lc8901a_Write (CAMACgpib, 9, 0, NSCA, 0);

	      GetCtrlVal (p1h, P1_TIME, &time_set ) ; /* read time (ms) */  
	      
		  SetCtrlAttribute (p1h, P1_TIMER, ATTR_ENABLED, 0);
    	  lc8901a_Write (CAMACgpib, 16, 0, NTGG, time_set);
    	  lc8901a_Write (CAMACgpib, 15, 0, NTGG, 0);
    	  																  
    	  SetCtrlVal (p1h, P1_LEDON,1);  
		  SetCtrlAttribute (p1h, P1_TIMER, ATTR_INTERVAL,time_set*0.001 );
    	  SetCtrlAttribute (p1h, P1_TIMER, ATTR_ENABLED, 1); 
    	  ResetTimer (p1h, P1_TIMER);
//		  Delay (0.1+(time_set/1000.));
   	 	}
    
    
		if ( rID==P1_COUNT ) {
		  sum = 0 ;
		  max = 100 ;
	  
 	      for ( ch=0 ; ch<16 ; ch ++ ) {
 	        data[ch]=0;
	        lc8901a_Read (CAMACgpib, 0, ch, NSCA, 1, 3000, &data[ch], &Xr, &Qr);
	        if (data[ch] > 0) data[ch]--;
	        if (data[ch] && 0x80000000) {
	            data[ch] &= 0x00FFFFFF;
	            //data[ch] |= 0x00800000;
    	    }
    	    sum += data[ch] ;
    	  }

	      SetCtrlVal (p1h, P1_I1, sum ) ;
      
		  plot_graph ( p1h, logscale, data ) ;
		}
    	if ( rID==P1_INIT ) {
	       init_CAMAC_GPIB ( CAMACgpib ) ;  
	    }
    
	    if ( rID==P1_SAVE ) {
	      if ( ( fp = fopen ( "scaler.txt", "w+" ) ) == NULL ) printf ( "ERROR" ) ;
	      for ( ch=0; ch<16 ; ch++ ) {  
 	        fprintf ( fp, "%d\t%d\n", ch, data[ch] ) ;
	      }
	      fclose ( fp ) ;
	    }

	    if ( rID==P1_APPEND ) {
		  GetCtrlVal (p1h, P1_APFILE, filename);

	      if ( ( fp = fopen ( filename, "a" ) ) == NULL ) printf ( "ERROR" ) ;
	      for ( ch=0; ch<16 ; ch++ ) {  
	        fprintf ( fp, "%d\t", data[ch] ) ;
	      }
	      fprintf ( fp, "\n" ) ;
	      fclose ( fp ) ;
	    }
    
		if ( rID==P1_REPLOT ) {
		  plot_graph ( p1h, logscale, data ) ;
		}
    
    
	    if ( rID == P1_PRINT ) {
	      SetCtrlVal (p1h, P1_L1,1);
		  PrintPanel (p1h, "", 1, VAL_VISIBLE_AREA, 1);
	      SetCtrlVal (p1h, P1_L1,0);
	    }


		if ( rID == P1_LOG_SCALE ) {
		  GetCtrlVal (p1h, P1_LOG_SCALE, &logscale) ;
	  
          plot_graph ( p1h, logscale, data ) ;
	    }		
    }
    
    if ( rph == p2h ) {
	    switch ( rID ) {
	      case P2_OFS1: GetCtrlVal (p2h, P2_OFS1, &os[0] ) ; i=0 ; break ;
	      case P2_OFS2: GetCtrlVal (p2h, P2_OFS2, &os[1] ) ; i=1 ; break ;
	      case P2_OFS3: GetCtrlVal (p2h, P2_OFS3, &os[2] ) ; i=2 ; break ;
	      case P2_OFS4: GetCtrlVal (p2h, P2_OFS4, &os[3] ) ; i=3 ; break ;
	      case P2_OFS5: GetCtrlVal (p2h, P2_OFS5, &os[4] ) ; i=4 ; break ;
	      case P2_OFS6: GetCtrlVal (p2h, P2_OFS6, &os[5] ) ; i=5 ; break ;
	      case P2_OFS7: GetCtrlVal (p2h, P2_OFS7, &os[6] ) ; i=6 ; break ;
	      case P2_OFS8: GetCtrlVal (p2h, P2_OFS8, &os[7] ) ; i=7 ; break ;
	      case P2_OFS9: GetCtrlVal (p2h, P2_OFS9, &os[8] ) ; i=8 ; break ;
	      case P2_OFS10: GetCtrlVal (p2h, P2_OFS10, &os[9] ) ; i=9 ; break ;
	    }
    
    
	    for ( ch=0 ; ch<32 ; ch ++ ) {  
	      sdata[i][ch]=data[ch];
	    }
    
    }
    
    GetUserEvent (1, &rph, &rID);
  }
      
      
  //lc8901a_close (CAMACgpib);
  return 0;
}

int CVICALLBACK TimerCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_TIMER_TICK:
								 
			    SetCtrlVal (panel, P1_LEDON,0); 
			    SetCtrlAttribute (panel, P1_TIMER, ATTR_ENABLED, 0); 
			break;
		}
	return 0;
}
